﻿'//////////////////////////////////////////////////////////////////////////////
' Author: SYO
' Copyright (c) 2012
' ScriptEZ.API based application which uses facilities from RAD API: GeolocateMe
' . Demoing a "Webservice Consumer"
' . Echo() method usage which displays always messages into a console window
'   instead of WScript.Echo() with 2 behaviours for cscript.exe or wscript.exe
' . ScriptEZ.API Remoting Control Interface usage by NPTelnet.exe program
'   (Connect to ScriptEZ.API process with <processid (PID)> as connection name)
'
' Purpose: Request for a Geolocalisation from a Web Service located at:
'          http://ipapi.co/{ip}/xml
'//////////////////////////////////////////////////////////////////////////////
Option Explicit Off

Module ModuleMain
    ' These below consts can be modified
    ' --begin
    Const METHOD = "POST"             ' "POST" or "GET"
    Const HEADER = "0"                ' size of header area in byte
    Const URL = "http://ipapi.co"
    ' POST method
    Const POST_VARS = ""   ' Each variable must be encoded with URLEncode()
    Const RAW_REMOTEFILENAME = "/{ip}/xml"
    ' GET method
    Const REMOTEFILENAME = "/{ip}/xml"
    ' --end

    Const SCRIPTEZ_API = "ScriptEZ.API" ' Component name
    Public WithEvents ScriptEZ As SCRIPTEZLib.API
    Public exitProcess As Long = 0

    ' Main entry of the app

    Sub Main()
        exitProcess = 0
        ' Detach a new thread for instantiating ScriptEZ.API component
        oThread = New Threading.Thread(AddressOf WorkerThread)

        ' explicit thread apartment affinity to MTA (Multi Threaded Apartment)
        ' [ aka CoInitializeEx(NULL, COINIT_MULTITHREADED) in C/C++]
        oThread.SetApartmentState(Threading.ApartmentState.MTA)

        ' start the thread
        oThread.Start()
        ' Endlessly loop until AppEventID = 3 sent from user with Remoting Control Interface
        Do Until exitProcess = 3
            Threading.Thread.Sleep(200)
        Loop
        oThread.Abort()

        '-- App unregister before ending
        ScriptEZ.AppUnregister()
        ScriptEZ = Nothing
    End Sub

    ' .NET thread procedure
    Public Sub WorkerThread()
        Dim httpFileHandle As Long = 0
        WshShell = CreateObject("WScript.Shell")
        fso = CreateObject("Scripting.FileSystemObject")

        '-- attempt to instantiate component object with callback interface
        'ScriptEZ = New SCRIPTEZLib.API 
        ScriptEZ = CreateObject(SCRIPTEZ_API)

        If ScriptEZ Is Nothing Then
            MsgBox("Please, download and register " + SCRIPTEZ_API +
                   " component from http://sovann.googlepages.com and try again...", vbInformation)
            Exit Sub
        End If

        ScriptEZ.ClearConsole()
        ScriptEZ.SetCPUCoreAffinity("0")

        '-- Register App for receiving AppEvent(s) with onAppEvent(EventID)
        ScriptEZ.AppRegister()

        Dim readTotalBytes, readBytes, xmlObject
        Dim xml_response
        readTotalBytes = 0
        xmlObject = 0

        Title = ""

        '-- http file opening and executing
        If (METHOD = "GET") Then
            '-- (1) using GET method for sending request 
            httpFileHandle = ScriptEZ.OpenHttpFile(URL, Replace(REMOTEFILENAME, "{ip}", GetMyPublicIP), CLng(HEADER))
            Title = REMOTEFILENAME
        ElseIf (METHOD = "POST") Then
            '-- (2) using POST method for sending request
            httpFileHandle = ScriptEZ.OpenHttpAndPost(URL, Replace(RAW_REMOTEFILENAME, "{ip}", GetMyPublicIP), POST_VARS, "", "", CLng(HEADER))
            Title = RAW_REMOTEFILENAME
        End If

        ' string format can be a fuss, because of accents from UTF-8  
        'xml_response = ""
        ' binary format is more relevant since it takes care of UTF-8
        xml_response = My.Application.Info.DirectoryPath & "\" & ScriptEZ.GetCurrentProcessId().ToString() & ".xml"
        If httpFileHandle > 0 Then
            '-- http file reading
            r = ScriptEZ.HttpReadFile(httpFileHandle)
            If r = 1 Then
                readBytes = ScriptEZ.HttpGetReadBytes(httpFileHandle)
                readTotalBytes = readTotalBytes + readBytes

                hFile = ScriptEZ.OpenBinaryFile(xml_response)
                If hFile > 0 Then
                    '-- if more than one reading operation, iterate into this loop
                    Do Until readBytes <= 0 '-- http file fetching data
                        ' string format can be a fuss, because of accents from UTF-8  
                        '         data_chunk = ScriptEZ.HttpGetReadBufferAsBSTR(httpFileHandle)
                        '         xml_response = xml_response + data_chunk
                        '         ScriptEZ.FreeBSTR data_chunk

                        ' binary format is more relevant since it takes care of UTF-8
                        r = ScriptEZ.WriteBinaryFile(hFile, ScriptEZ.HttpGetReadBufferAsBinary(httpFileHandle), readBytes)

                        If readTotalBytes < 1024 Then
                            ScriptEZ.SetConsoleTitle(CStr(FormatNumber(readTotalBytes, 0, 0, -2)) + " Bytes read from " +
                        URL + Title + " by HTTP/" + METHOD)
                        Else
                            ScriptEZ.SetConsoleTitle(CStr(FormatNumber(readTotalBytes / 1024, 0, 0, -2)) +
                                                     " KBytes (" + CStr(readTotalBytes) + " Bytes) read from " +
                                                     URL + Title + " by HTTP/" + METHOD)
                        End If
                        '-- http file reading    
                        r = ScriptEZ.HttpReadFile(httpFileHandle)
                        readBytes = ScriptEZ.HttpGetReadBytes(httpFileHandle)
                        readTotalBytes = readTotalBytes + readBytes
                    Loop
                End If

                ScriptEZ.CloseBinaryFile(hFile)
            End If
            '-- http file closing
            ScriptEZ.CloseHttpFile(httpFileHandle)
        End If

        If httpFileHandle > 0 Then
            ScriptEZ.ClearConsole()
            '-- displaying data here....
            If xmlObject <= 0 Then
                xmlObject = ScriptEZ.CreateXMLObject(xml_response)
            Else
                r = ScriptEZ.PopulateXMLObject(xmlObject, xml_response)
            End If
            If xmlObject > 0 Then
                IP = ScriptEZ.NamedItemValueFromElement(xmlObject, "ip", "<text/>")
                CountryName = ScriptEZ.NamedItemValueFromElement(xmlObject, "country_name", "<text/>")
                zTimeZone = ScriptEZ.NamedItemValueFromElement(xmlObject, "timezone", "<text/>")
                City = ScriptEZ.NamedItemValueFromElement(xmlObject, "city", "<text/>")
                Region = ScriptEZ.NamedItemValueFromElement(xmlObject, "region", "<text/>")
                Organization = ScriptEZ.NamedItemValueFromElement(xmlObject, "org", "<text/>")
                ScriptEZ.Echo(vbLf + " . Own Memory in use  : " + CStr(FormatNumber(ScriptEZ.ProbeMemoryStatus(7) / 1024, 2)) + " KBytes" + vbLf)
                ScriptEZ.Echo(vbLf + " . Your GeoIdentity is")
                ScriptEZ.Echo(vbLf + "   Public IP..........: " + IIf(IP.ToString().IndexOf(":") > 0, IP + " (" + GetMyPublicIP("http://myip.dnsomatic.com") + ")", IP))
                ScriptEZ.Echo(vbLf + "   CountryName........: " + CountryName + " (" + zTimeZone + ")")
                ScriptEZ.Echo(vbLf + "   City...............: " + City)
                ScriptEZ.Echo(vbLf + "   Region.............: " + Region)
                ScriptEZ.Echo(vbLf + "   Internet Provider..: " + Organization)
                fso.DeleteFile(xml_response)
                ScriptEZ.DestroyXMLObject(xmlObject)
            End If
        End If
    End Sub

    Function GetMyPublicIP(Optional ByVal urlWebService As String = "http://bot.whatismyipaddress.com")
        '-- using GET method for sending request 
        local_httpFileHandle = ScriptEZ.OpenHttpFile(urlWebService, "", 0)
        response = "n.a"
        If local_httpFileHandle > 0 Then
            '-- http file reading
            r = ScriptEZ.HttpReadFile(local_httpFileHandle)
            If r = 1 Then
                read = ScriptEZ.HttpGetReadBytes(local_httpFileHandle)
                '-- http file fetching ip string as xxx.xxx.xxx.xxx
                data_chunk = ScriptEZ.HttpGetReadBufferAsBSTR(local_httpFileHandle)
                response = data_chunk
                ScriptEZ.FreeBSTR(data_chunk)
            End If

            '-- http file closing
            ScriptEZ.CloseHttpFile(local_httpFileHandle)
        End If
        GetMyPublicIP = response
    End Function



    Private Function ScriptEZ_AppEventHelp() As String Handles ScriptEZ.AppEventHelp

        ScriptEZ_AppEventHelp = "Type <3> to quit app\n"

    End Function



    Private Sub ScriptEZ_onAppEvent(EventID As Integer) Handles ScriptEZ.onAppEvent

        If EventID = 3 Then ScriptEZ.Terminate(0)

    End Sub

End Module
