﻿Imports System
Imports System.Threading
'//////////////////////////////////////////////////////////////////////////////
' Author: SY
' Demo ScriptEZ.API based application which uses facilities from RAD API:
' . TCP/IP usage
' . Threadpool usage
' . Echo() method usage which displays always messages into a console window
' . ScriptEZ.API Remoting Control Interface usage by NPTelnet.exe program
'   (Connect to ScriptEZ.API process with <processid (PID)> as connection name)
'
' Purpose: Simple TCP/IP Server broadcasting random data packet 2.5 times/second
'//////////////////////////////////////////////////////////////////////////////
Module ScriptEZSvr
    Dim oThread As Thread
    Dim WithEvents ScriptEZ As SCRIPTEZLib.API
    Dim WithEvents ScriptEZConsole As SCRIPTEZLib.API
    Dim hSocketSvr As Long
    Dim StartTime
    Dim TotalRecv
    Dim hThreadPool
    Dim hThreadPoolConsole
    Dim onExit
    Dim showRecvData
    Dim enumerateClient
    Sub Main()

        ' Detach a new thread for instantiating ScriptEZ.API component
        oThread = New Thread(AddressOf WorkerThread)

        ' explicit thread apartment affinity to MTA (Multi Threaded Apartment)
        ' [ aka CoInitializeEx(NULL, COINIT_MULTITHREADED) in C/C++]
        'oThread.SetApartmentState(ApartmentState.MTA)

        ' start the thread
        oThread.Start()

        ' Endlessly loop until AppEventID = 3 sent from user with Remoting Control Interface
        Do Until onExit = 3
            Thread.Sleep(200)
        Loop
        oThread.Abort()
        ScriptEZ.AppUnregister()
        ScriptEZ = Nothing
        ScriptEZConsole = Nothing

    End Sub
    ' .NET thread procedure
    Public Sub WorkerThread()

        Dim r
        enumerateClient = 0
        showRecvData = 0
        onExit = 0
        TotalRecv = 0
        StartTime = Now()
        ' instantiate component object with callback interface
        ScriptEZ = New SCRIPTEZLib.API
        ScriptEZConsole = New SCRIPTEZLib.API
        'ScriptEZ.SetCPUCoreAffinity "0"

        ' Register App for receiving AppEvent(s) with onAppEvent(EventID)
        ScriptEZ.AppRegister()

        ' Hide console window if 0
        ScriptEZ.ShowConsoleWindow(1)

        ' create server socket object +  extra  encryption
        hSocketSvr = ScriptEZ.CreatePoolSocketServerObject(12346, vbNullString, 100, 8, 0, 0)

        If hSocketSvr > 0 Then
            ScriptEZ.EnableSocketObjectDataEncryption(hSocketSvr, True, -1)

            ' creating thread pool with 4 threads
            hThreadPool = ScriptEZ.CreateTaskQueueObject(4)

            hThreadPoolConsole = ScriptEZConsole.CreateTaskQueueObject(1)
            r = ScriptEZConsole.AddTask(hThreadPoolConsole, 0)
        End If

    End Sub
    ' Callbacks for App Events notification and fetching help text
    '/////////////////////////////////////////////////////////////
    Private Function ScriptEZ_AppEventHelp() As String Handles ScriptEZ.AppEventHelp
        ScriptEZ_AppEventHelp = _
         CStr("  **** Supported AppEvents ****" + vbLf + _
              "AppEventID = 0" + vbTab + "to hide received data (default)" + vbLf + _
              "AppEventID = 1" + vbTab + "to display received data" + vbLf + _
              "AppEventID = 3" + vbTab + "to exit" + vbLf + _
              "AppEventID = 4" + vbTab + "to enumerate connected clients" + vbLf + _
              "AppEventID = 5" + vbTab + "not to enumerate connected clients (default)" + vbLf _
              )
    End Function
    Private Sub ScriptEZ_onAppEvent(ByVal EventID As Integer) Handles ScriptEZ.onAppEvent

        If EventID = 3 Then onExit = EventID

        If EventID = 0 Then showRecvData = EventID
        If EventID = 1 Then showRecvData = EventID
        If EventID = 4 Then enumerateClient = 1
        If EventID = 5 Then enumerateClient = 0

    End Sub
    ' Callbacks for socket object 
    ' ( server side )
    '///////////////////////////
    Private Function ScriptEZ_onConnectionRequest(ByVal hSocketClientObjectProxy As Integer) As Integer Handles ScriptEZ.onConnectionRequest
        Dim r
        ScriptEZ_onConnectionRequest = CLng(ScriptEZ.AddSocketClientObjectEventHandlers(hSocketClientObjectProxy))
        ' delegating to the thread pool....
        r = ScriptEZ.AddTask(hThreadPool, hSocketClientObjectProxy)
    End Function
    ' ( client/proxy side)
    '/////////////////////
    Private Function ScriptEZ_onConnectionState(ByVal hOwnerSocket As Integer, ByVal state_desc As String, ByVal error_number As Integer) As Integer Handles ScriptEZ.onConnectionState
        ScriptEZ.Echo(state_desc)
        ScriptEZ_onConnectionState = CLng(0)
    End Function
    ' ( client/proxy side)
    '/////////////////////
    Private Function ScriptEZ_onData(ByVal hOwnerSocket As Integer, ByVal raw_data As Integer, ByVal data_size As Integer) As Integer Handles ScriptEZ.onData
        Dim bstrData
        bstrData = ScriptEZ.ConvertStringToBSTR(raw_data)
        TotalRecv = TotalRecv + data_size
        If showRecvData = 1 Then
            ScriptEZ.Echo(vbLf + CStr(Now) + " size(" + CStr(data_size) + "): " + CStr(bstrData))
        End If
        ScriptEZ.FreeString(raw_data)
        ScriptEZ_onData = CLng(0)
    End Function
    ' Callbacks for thread pool object (TaskQueueObject)
    '//////////////////////////////////////////////////
    Private Function ScriptEZ_onRun(ByVal hSocketClientProxy As Integer) As Integer Handles ScriptEZ.onRun
        Dim TestData
        Dim r
        ' Send random data 2.5 times per second
        ScriptEZ.Sleep(400)
        r = Rnd()
        If (r = 0) Then r = 1
        TestData = "BoM" + StrDup(CInt(r * 1536), "x") + "EoM" + vbLf + Chr(0)
        r = ScriptEZ.SendSocketObjectData(hSocketClientProxy, TestData, Len(TestData))
        If onExit = 3 Or r <> 0 Then
            ScriptEZ_onRun = CLng(0)
        Else
            ScriptEZ_onRun = CLng(1)
        End If
    End Function

    Private Function ScriptEZConsole_onRun(ByVal task_local_storage As Integer) As Integer Handles ScriptEZConsole.onRun

        Dim zClient
        Dim hSocketClientProxy
        ScriptEZ.SetConsoleTitle(CStr(StartTime) + " - TCP/IP .NET Server@12346 | Own memory: " + _
                                 CStr(FormatNumber(ScriptEZ.ProbeMemoryStatus(7) / 1024, 2, , , TriState.True)) + " KBytes")

        ScriptEZConsole.Sleep(1000)
        ScriptEZ.ClearConsole()
        If enumerateClient = 0 Then
            ScriptEZ.Echo(vbLf + " Clients : " + _
                              CStr(ScriptEZ.GetActiveSocketClientObjectNumber(hSocketSvr)) + vbLf + " Received: " + _
                              CStr(FormatNumber(TotalRecv / 1024.0, 2, , , TriState.True)) + " KBytes")
        End If

        If enumerateClient = 1 Then
            ScriptEZ.Echo(vbLf + " Clients : " + _
                          CStr(ScriptEZ.GetActiveSocketClientObjectNumber(hSocketSvr)) + vbLf + " Received: " + _
                          CStr(FormatNumber(TotalRecv / 1024.0, 2, , , TriState.True)) + " KBytes")
            Do While 1
                hSocketClientProxy = ScriptEZ.EnumActiveSocketClientObject(hSocketSvr)
                If hSocketClientProxy <= 0 Then
                    Exit Do
                End If
                zClient = ScriptEZ.GetSocketConnectedClientName(hSocketClientProxy)
                ScriptEZ.Echo(vbLf + _
                       CStr(zClient) + _
                       " Inbound:" + CStr(ScriptEZ.SocketObjectPendingInboundMessageCount(hSocketClientProxy)) + _
                       " Outbound:" + CStr(ScriptEZ.SocketObjectPendingOutboundMessageCount(hSocketClientProxy)))
            Loop
            If ScriptEZ.GetActiveSocketClientObjectNumber(hSocketSvr) = 0 Then TotalRecv = 0
        End If

        If onExit = 3 Then
            ScriptEZConsole_onRun = CLng(0)
        Else
            ScriptEZConsole_onRun = CLng(1)
        End If
    End Function
End Module
