﻿Imports System
Imports System.Threading
'//////////////////////////////////////////////////////////////////////////////
' Author: SY
' Demo ScriptEZ.API based application which uses facilities from RAD API:
' . TCP/IP usage
' . Threadpool usage
' . Echo() method usage which displays always messages into a console window
'   instead of WScript.Echo() with 2 behaviours for cscript.exe or wscript.exe
' . ScriptEZ.API Remoting Control Interface usage by NPTelnet.exe program
'   (Connect to ScriptEZ.API process with <processid (PID)> as connection name)
'
' Purpose: Simple TCP/IP Client receiving random packets and sending back at
'          every eight received packets
'//////////////////////////////////////////////////////////////////////////////
Module ScriptEZ

    Dim WithEvents ScriptEZMain As SCRIPTEZLib.API
    Dim hSocketClient
    Dim ReplyCount
    Dim StartTime
    Dim ReadyToSend
    Dim TotalRecv
    Dim TotalSend
    Dim LastRecvTime
    Dim Err_num
    Dim hThreadPool
    Dim onExit
    Dim InputString
    Dim showRecvData
    Dim Resend
    Dim ScrollOutput
    Dim PendingTasks
    Dim Socket_ver
    Dim bstrData
    Dim oThread As Thread

    ' Main Subroutine
    '////////////////
    Sub Main()

        Dim r
        ' Detach a new thread for instantiating ScriptEZ.API component
        oThread = New Thread(AddressOf WorkerThread)

        ' explicit thread apartment affinity to MTA (Multi Threaded Apartment)
        ' [ aka CoInitializeEx(NULL, COINIT_MULTITHREADED) in C/C++]
        'oThread.SetApartmentState(ApartmentState.MTA)

        ' start the thread
        oThread.Start()

        ' Endlessly loop until AppEventID = 3 sent from user with Remoting Control Interface
        Do Until onExit = 3
            Thread.Sleep(200)
        Loop

        ' empty queued msg for sending
        r = ScriptEZMain.SendSocketObjectData(hSocketClient, vbNullString, 0)

        ' App unregister before ending
        ScriptEZMain.AppUnregister()
        Thread.Sleep(4000)
        ScriptEZMain = Nothing

        oThread.Abort()

    End Sub
    ' .NET thread procedure
    Public Sub WorkerThread()

        Dim HostAndPort

        ScrollOutput = 1
        Resend = 0
        showRecvData = 0
        onExit = 0
        ReplyCount = 0
        ReadyToSend = 0

        TotalRecv = 0
        TotalSend = 0

        StartTime = Now()
        LastRecvTime = Now()

        ' instantiate component object with callback interface
        ScriptEZMain = New SCRIPTEZLib.API
        'ScriptEZMain.SetCPUCoreAffinity "0"

        ' create client socket object + xtra encryption
        hSocketClient = ScriptEZMain.CreateSocketClientObjectEx
        ScriptEZMain.EnableSocketObjectDataEncryption(hSocketClient, True, -1)

        ' connect to server
        InputString = InputBox("Enter your hostname and port, such like localhost:12346", _
                               "Choose a hostname", _
                               "localhost:12346")
        If Len(InputString) <= 0 Then
            onExit = 3
            Exit Sub
        End If
        HostAndPort = Split(InputString, ":")

        ' creating thread pool with 4 threads for polling info 
        hThreadPool = ScriptEZMain.CreateTaskQueueObject(4)

        Err_num = ScriptEZMain.ConnectSocketServerObject(HostAndPort(0), HostAndPort(1), hSocketClient)
        If Err_num <> 0 Then Exit Sub

        Socket_ver = ScriptEZMain.GetSocketObjectVersion(hSocketClient)

        ' Register App for receiving AppEvent(s) with onAppEvent(EventID)
        ScriptEZMain.AppRegister()


    End Sub
    Public Function DataHandler()

        Dim data_size
        data_size = Len(bstrData)
        If showRecvData = 1 Then ScriptEZMain.Echo(CStr(Now) + " size(" + CStr(data_size) + "): " + CStr(bstrData))

        If Resend = 1 Then
            Err_num = ScriptEZMain.SendSocketObjectData(hSocketClient, bstrData, data_size)
            TotalSend = TotalSend + data_size
            ReplyCount = ReplyCount + 1
        Else
            TotalSend = 0
        End If
        DataHandler = 0

    End Function
    ' Callbacks for App Events notification and fetching help text
    '/////////////////////////////////////////////////////////////
    Private Function ScriptEZMain_AppEventHelp() As String Handles ScriptEZMain.AppEventHelp
        ScriptEZMain_AppEventHelp = vbLf + _
        CStr("Filename: " + Chr(34) + CStr(My.Application.Info.ProductName) + Chr(34) + vbLf + _
             "  **** Supported AppEvents ****" + vbLf + _
             "AppEventID = 0" + vbTab + "to hide received data (default)" + vbLf + _
             "AppEventID = 2" + vbTab + "to not resend back data (default)" + vbLf + _
             "AppEventID = 6" + vbTab + "to enable scrolling (default)" + vbLf + _
             "AppEventID = 1" + vbTab + "to display received data" + vbLf + _
             "AppEventID = 4" + vbTab + "to start resending back data" + vbLf + _
             "AppEventID = 5" + vbTab + "to disable scrolling" + vbLf + _
             "AppEventID = 3" + vbTab + "to exit" + vbLf _
         )
    End Function

    Private Sub ScriptEZMain_onAppEvent(ByVal EventID As Integer) Handles ScriptEZMain.onAppEvent
        ScriptEZMain.Echo(" -> EventID:" + CStr(EventID))
        Select Case EventID
            Case 3
                onExit = 3
            Case 0
                showRecvData = 0
            Case 1
                showRecvData = 1
            Case 2
                Resend = 0
            Case 4
                Resend = 1
            Case 5
                ScrollOutput = 0
            Case 6
                ScrollOutput = 1
        End Select
    End Sub
    ' Callbacks for socket object 
    ' ( client/proxy side)
    '///////////////////////////
    Private Function ScriptEZMain_onConnectionState(ByVal hOwnerSocket As Integer, ByVal state_desc As String, ByVal error_number As Integer) As Integer Handles ScriptEZMain.onConnectionState

        Dim r
        ScriptEZMain.Echo(state_desc)
        Err_num = error_number
        If error_number = 0 Then
             r = ScriptEZMain.AddTask(hThreadPool, hOwnerSocket)
        End If
        ScriptEZMain_onConnectionState = CLng(0)

    End Function
    ' ( client/proxy side)
    '//////////////////////////
    Private Function ScriptEZMain_onData(ByVal hOwnerSocket As Integer, ByVal raw_data As Integer, ByVal data_size As Integer) As Integer Handles ScriptEZMain.onData

        Dim r
        ' converting string to BSTR
        bstrData = ScriptEZMain.ConvertStringToBSTR(raw_data)
        LastRecvTime = Now()
        TotalRecv = TotalRecv + data_size

        If showRecvData = 0 Then
            ScriptEZMain.Echo(" ThreadID:" + CStr(ScriptEZMain.GetCurrentThreadId()) + _
            vbTab + " Data size..:" + CStr(data_size))
        End If

        ReadyToSend = ReadyToSend + 1
        If ReadyToSend = 8 Then 'sending back every 8 received packets
            ' Handle this packet of data
            r = DataHandler()
            ReadyToSend = 0
        End If
        ScriptEZMain.FreeString(raw_data)
        ScriptEZMain_onData = CLng(0)

    End Function
    ' Callback for thread pool object (TaskQueueObject)
    '//////////////////////////////////////////////////
    Private Function ScriptEZMain_onRun(ByVal task_local_storage As Integer) As Integer Handles ScriptEZMain.onRun

        If Err_num <> 0 Or onExit = 3 Then
            ScriptEZMain_onRun = CLng(0) ' stop this task
            Exit Function
        Else
            ScriptEZMain_onRun = CLng(1) ' run again this task
        End If

        Thread.Sleep(1000)
        Dim hSocket
        Dim Total

        hSocket = task_local_storage
        ScriptEZMain.SetConsoleTitle(CStr(StartTime) + " - TCP/IP .NET Client | sentback(" + _
                                     CStr(ReplyCount) + ")/" + InputString)
        ScriptEZMain.PeekSocketClientObjectStatsOnReceivedData(hSocket)
        PendingTasks = ScriptEZMain.GetPendingTaskCount(hThreadPool)
        If ScrollOutput = 0 Then ScriptEZMain.ClearConsole()
        Total = ScriptEZMain.SocketSplittedPackets + _
                ScriptEZMain.SocketOverlappedPackets + _
                ScriptEZMain.SocketSimplePackets + _
                ScriptEZMain.SocketWrongPackets
        If Total > 0 Then
            ScriptEZMain.Echo( _
             " ." + CStr(Now) + vbLf + " (WinSock version " + Socket_ver + ")" + vbLf + vbLf + _
             " Own in use memory: " + FormatNumber(ScriptEZMain.ProbeMemoryStatus(7) / 1024.0, 0, 0, -2) + " KBytes" + vbLf + _
             " Inbound..........: " + CStr(ScriptEZMain.SocketObjectPendingInboundMessageCount(hSocket)) + _
                               " (" + FormatNumber(TotalRecv / 1024, 0, 0, -2) + " KBytes)" + vbLf + _
             " Outbound.........: " + CStr(ScriptEZMain.SocketObjectPendingOutboundMessageCount(hSocket)) + _
                               " (" + FormatNumber(TotalSend / 1024, 0, 0, -2) + " KBytes)" + vbLf + _
             " Pending..........: " + CStr(PendingTasks) + vbLf + vbLf + " ." + CStr(LastRecvTime) + vbLf + _
             " Splitted.........: " + CStr(ScriptEZMain.SocketSplittedPackets) + _
                               " (" + FormatNumber((ScriptEZMain.SocketSplittedPackets / Total) * 100.0, 0, 0, -2) + "%)" + vbLf + _
             " Overlapped.......: " + CStr(ScriptEZMain.SocketOverlappedPackets) + _
                               " (" + CStr(FormatNumber((ScriptEZMain.SocketOverlappedPackets / Total) * 100.0, 0, 0, -2)) + "%)" + vbLf + _
             " Simple...........: " + CStr(ScriptEZMain.SocketSimplePackets) + _
                               " (" + CStr(FormatNumber((ScriptEZMain.SocketSimplePackets / Total) * 100.0, 0, 0, -2)) + "%)" + vbLf + _
             " Wrong............: " + CStr(ScriptEZMain.SocketWrongPackets) + vbLf + _
             " PooledThreadId...: " + CStr(ScriptEZMain.GetCurrentPooledThreadId) + vbLf)
        End If
    End Function

End Module
