'//////////////////////////////////////////////////////////////////////////////
' Author: SY
' Demo ScriptEZ.API based application which uses facilities from RAD API:
' . TCP/IP usage
' . Encryption
' . Threadpool usage
' . Echo() method usage which displays always messages into a console window
'   instead of WScript.Echo() with 2 behaviours for cscript.exe or wscript.exe
' . ScriptEZ.API Remoting Control Interface usage by NPTelnet.exe program
'   (Connect to ScriptEZ.API process with <processid (PID)> as connection name)
'
' Purpose: TCP/IP Server for remote copying out a video file
'//////////////////////////////////////////////////////////////////////////////

const source_file  = "C:\Users\Public\Videos\Sample Videos\Wildlife.wmv"
const SCRIPTEZ_API = "ScriptEZ.API" ' Component name

Dim ScriptEZ
Dim hSocketSvr
Dim hThreadPool
Dim onExit
Dim hFileBinary

' start the script
'////////////////
'On Error Resume Next
Main()

' Main Subroutine
'////////////////
Sub Main()

  onExit         = 0

  Set WshShell = CreateObject("WScript.Shell")
  Set fso      = CreateObject("Scripting.FileSystemObject")
  
  Dim Windir
  Dim FullQualifiedEngineName
  ' if Win64, restart with WScript.exe/CScript.exe 32 bits
  On Error Resume Next
  Windir = WshShell.ExpandEnvironmentStrings("%WinDir%")
  FullQualifiedEngineName = Windir+"\SysWOW64" + Mid(WScript.FullName,InStrRev(WScript.FullName,"\"),Len(WScript.FullName))
  If fso.FolderExists(Windir+"\SysWOW64") And _ 
     LCase(WScript.FullName) <>  LCase(FullQualifiedEngineName) And _
     LCase(WScript.FullName) = LCase(Windir+"\System32" + Mid(WScript.FullName,InStrRev(WScript.FullName,"\"),Len(WScript.FullName))) And _
     fso.FileExists(FullQualifiedEngineName) = True Then
    WshShell.Run FullQualifiedEngineName + " " + Chr(34) + CStr(WScript.ScriptFullName) + Chr(34)
    Exit Sub
  End If
  
  ' instantiate component object with callback interface
  Set ScriptEZ = WScript.CreateObject(SCRIPTEZ_API,"ScriptEZ_")
  ScriptEZ.SetCPUCoreAffinity "0"

  ' Register App for receiving AppEvent(s) with onAppEvent(EventID)
  ScriptEZ.AppRegister

  ' create server socket object with one client at once
  hSocketSvr = ScriptEZ.CreatePoolSocketServerObject(12349,vbNullString,1,4,0,0)

  If hSocketSvr > 0 Then
    'no encryption: ScriptEZ.EnableSocketObjectDataEncryption hSocketSvr,true,-1 

    ' creating thread pool with 1 thread
    hThreadPool = ScriptEZ.CreateTaskQueueObject(1)

    ScriptEZ.SetConsoleTitle "Demo File Server - in use RAM: " + _ 
                             CStr(ScriptEZ.ProbeMemoryStatus(7)) + " KBytes"    
    ' block until AppEventID = 3 is received
    Do Until onExit = 3
       WScript.Sleep 1000
    Loop
  End If

  ScriptEZ.AppUnregister
  WScript.Sleep 4000
  Set ScriptEZ = Nothing
End Sub

' Callbacks for App Events notification and fetching help text
'/////////////////////////////////////////////////////////////
Sub ScriptEZ_onAppEvent(EventID)
  If EventID = 3 Then onExit = EventID
End Sub

Function ScriptEZ_AppEventHelp()
  ScriptEZ_AppEventHelp = CStr("  **** Supported AppEvents ****" + vbLF + _
                               "AppEventID = 3" + vbTab + "to exit" )
End Function

' Callbacks for socket object 
' ( server side )
'///////////////////////////
Function ScriptEZ_onConnectionRequest(hSocketClientObjectProxy) 

 ScriptEZ_onConnectionRequest = CLng(1)
 r = ScriptEZ.EnableSocketObjectInOutBoundBandwidthPolling(hSocketClientObjectProxy)
 ' delegating to the thread pool....
 hFileBinary = ScriptEZ.OpenBinaryFile(source_file)
 onExit = 0
 r = ScriptEZ.AddTask(hThreadPool,hSocketClientObjectProxy)

End Function

' ( client/proxy side)
'/////////////////////
Function ScriptEZ_onConnectionState(hOwnerSocket,state_desc,error_number)
  ScriptEZ.Echo state_desc
  onExit = error_number
  ScriptEZ_onConnectionState = CLng(0)
End Function 

' Callbacks for thread pool object (TaskQueueObject)
'//////////////////////////////////////////////////
Function ScriptEZ_onRun( hSocketClientProxy )

    Dim bsData, totalSize
    totalSize = 0
    timeout   = 15000/50
    timeCount = 0
    Do Until onExit <> 0
      If ScriptEZ.SocketObjectPendingOutboundMessageCount(hSocketClientProxy) = 0 Then
          pData    = ScriptEZ.ReadBinaryFile(hFileBinary)
          dataSize = ScriptEZ.ReadBinaryBytes(hFileBinary)
          If pData > 0 And  dataSize > 0 Then
            totalSize = totalSize + dataSize 
            ' 16384 is maxi lenght
            bsData = ScriptEZ.BinaryToBSTR(pData,dataSize)
            s = Len(bsData)

            r = ScriptEZ.SendSocketObjectData(hSocketClientProxy,String(5-Len(CStr(dataSize)),"0")+ CStr(dataSize), 5)
            r = ScriptEZ.SendSocketObjectData(hSocketClientProxy,bsData, s)
            If r <> 0 Then Exit Do ' connection broken or closed
          Else
            ' end of file
            r = ScriptEZ.SendSocketObjectData(hSocketClientProxy,"00000", Len("00000"))
            Exit Do
          End If
      End If
      WScript.Sleep 50
      timeCount = timeCount + 1
      If timeCount = timeout Then
        ScriptEZ.SetConsoleTitle "Demo File Server - in use RAM: " + CStr(ScriptEZ.ProbeMemoryStatus(7)) + _
                                 " KBytes | sent: " +  CStr(totalSize/1024) + " KBytes"
        ScriptEZ.ClearConsole
        hProxy = ScriptEZ.EnumActiveSocketClientObject(hSocketSvr)
        Do Until hProxy = 0
         bandwidth = ScriptEZ.GetSocketObjectInOutBoundBandwidth(hProxy)
         connectedClient = ScriptEZ.GetSocketConnectedClientName(hProxy)
         ScriptEZ.Echo connectedClient + " -> " + bandwidth
         hProxy = ScriptEZ.EnumActiveSocketClientObject(hSocketSvr)
         ScriptEZ.FreeBSTR bandwidth
        Loop
        timeCount = 0 
      End If
    Loop
    ScriptEZ.CloseBinaryFile hFileBinary
    ScriptEZ_onRun = CLng(0) ' end this task
  
End Function
